package com.ruoyi.oa.mapper;

import java.util.List;
import com.ruoyi.oa.domain.CmcCarApproval;

/**
 * cmc用车审批Mapper接口
 * 
 * @author cmc
 * @date 2024-02-23
 */
public interface CmcCarApprovalMapper 
{
    /**
     * 查询cmc用车审批
     * 
     * @param carApplyId cmc用车审批主键
     * @return cmc用车审批
     */
    public CmcCarApproval selectCmcCarApprovalByCarApplyId(String carApplyId);

    /**
     * 查询cmc用车审批列表
     * 
     * @param cmcCarApproval cmc用车审批
     * @return cmc用车审批集合
     */
    public List<CmcCarApproval> selectCmcCarApprovalList(CmcCarApproval cmcCarApproval);

    /**
     * 新增cmc用车审批
     * 
     * @param cmcCarApproval cmc用车审批
     * @return 结果
     */
    public int insertCmcCarApproval(CmcCarApproval cmcCarApproval);

    /**
     * 修改cmc用车审批
     * 
     * @param cmcCarApproval cmc用车审批
     * @return 结果
     */
    public int updateCmcCarApproval(CmcCarApproval cmcCarApproval);

    /**
     * 删除cmc用车审批
     * 
     * @param carApplyId cmc用车审批主键
     * @return 结果
     */
    public int deleteCmcCarApprovalByCarApplyId(String carApplyId);

    /**
     * 批量删除cmc用车审批
     * 
     * @param carApplyIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteCmcCarApprovalByCarApplyIds(String[] carApplyIds);
}
